unit ttdptxt;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Grids, ExtCtrls, AppEvnts;

type
  TInGameText = class(TForm)
    StringGrid1: TStringGrid;
    Help: TButton;
    Cancel: TButton;
    Save: TButton;
    PreviewTimer: TTimer;
    ApplicationEvents1: TApplicationEvents;
    CodeLab: TLabel;
    procedure FormResize(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure PreviewTimerTimer(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure ApplicationEvents1Activate(Sender: TObject);
    procedure ApplicationEvents1Deactivate(Sender: TObject);
    procedure SaveClick(Sender: TObject);
    procedure StringGrid1SelectCell(Sender: TObject; ACol, ARow: Integer;
      var CanSelect: Boolean);
    procedure StringGrid1KeyPress(Sender: TObject; var Key: Char);
    procedure StringGrid1SetEditText(Sender: TObject; ACol, ARow: Integer;
      const Value: String);
    procedure StringGrid1Exit(Sender: TObject);
    procedure HelpClick(Sender: TObject);
  private
    { Private declarations }
    DatUpdated:boolean;
    PatchDir,TxtName,DatName,NewName,ExeName:string;
    function RunCompPrg:boolean;
    procedure RunInit(var msg:TMessage); message WM_USER;
    procedure ReadTexts;
    procedure WriteTexts;
    function CheckForNewFile:boolean;
    procedure WMSysCommand(var Msg:TWMSysCommand); message WM_SYSCOMMAND;
  public
    OrigString:string;
    function CheckCError:boolean;
    { Public declarations }
  end;

var
  InGameText: TInGameText;

implementation

uses runmkpttxt, FoProgi, CompError, previewf;

{$R *.DFM}

procedure TInGameText.FormResize(Sender: TObject);
begin
 with StringGrid1 do
  ColWidths[1]:=ClientWidth-ColWidths[0]-5;
end;

procedure TInGameText.FormShow(Sender: TObject);
begin
 PatchDir:=ExtractFilePath(PatchPath);
 TxtName:=PatchDir+'ttdpttxt.txt';
 NewName:=PatchDir+'ttdpttxt.new';
 DatName:=PatchDir+'ttdpttxt.dat';
 ExeName:=PatchDir+'mkpttxt.exe';
 PostMessage(Handle,WM_USER,0,0);
end;

procedure TInGameText.ReadTexts;
var f:TextFile;
    s,s2,id:string;
    i,j,k,p,value,error:integer;
label EndOfLoop;

 function ShowError(const ErrStr:string):boolean;
  begin
   Result:=true;
   Case Application.MessageBox(PChar('There is an error in ttdpttxt.txt in line '+IntToStr(j)+':'#10#13+
                                     ErrStr+#10#13+'Do you want to ignore the error and continue?'#10#13+
                                     '(Choosing No will close the in-game translator window)'),
                                     'Error in data file',MB_ICONERROR or MB_YESNO) of
    ID_YES: Result:=false;
    ID_NO: ModalResult:=mrCancel;
   end;
  end;

begin
 StringGrid1.RowCount:=2;
 AssignFile(f,TxtName);
 Reset(f);
 i:=1; j:=0;
 while not Eof(f) do
  begin
   Readln(f,s);
   inc(j);
   if Trim(s)='' then continue;
   case s[1] of
    ' ',#9: s:=trim(s);
    '/': if (Length(s)>1) and (s[2]='/') then continue;
    else
     begin
      p:=Pos('=',s);
      if p=0 then
       begin
        if ShowError('Not an assigment (equals sign missing)') then Exit else continue;
       end;
      if s2<>'' then
       begin
        StringGrid1.RowCount:=i+1;
        StringGrid1.Cells[0,i]:=id;
        if FoForm.CStyle.Checked then s2:=FoForm.MakeCStyleString(s2);
        StringGrid1.Cells[1,i]:=s2;
        inc(i); s2:='';
       end;
      id:=Copy(s,1,p-1);
      s:=Copy(s,p+1,Length(s)-p);
      s:=Trim(s);
     end;
   end;
   if (s='') or (s[1]<>'"') then
    begin
     if ShowError('Text not in quotes') then Exit else continue;
    end;
   Delete(s,1,1);
   while (Length(s)>0) and (s[length(s)]<>'"') do Delete (s,Length(s),1);
   if s='' then
    begin
     if ShowError('Text not in quotes') then Exit else continue;
    end;
   Delete(s,Length(s),1);
   k:=1;
   while k<=length(s) do
    begin
     if s[k]='\' then
      begin
       if k=length(s) then
        begin
         if ShowError('Backslash without control char at the end of the line') then Exit else goto EndOfLoop;
        end
       else
        case s[k+1] of
         '"': begin s2:=s2+'"'; inc(k,2); end;
         'n': begin s2:=s2+#13; inc(k,2); end;
         '\': begin s2:=s2+'\'; inc(k,2); end;
         else
          begin
           Val('$'+Copy(s,k+1,2),value,Error);
           if Error<>0 then
            begin
             if ShowError('Incorrect backslash-sequence') then Exit else goto EndOfLoop;
            end;
           if value=0 then
            begin
             StringGrid1.RowCount:=i+1;
             StringGrid1.Cells[0,i]:=id; id:='';
             if FoForm.CStyle.Checked then s2:=FoForm.MakeCStyleString(s2);
             StringGrid1.Cells[1,i]:=s2;
             inc(i); s2:='';
            end
           else
            s2:=s2+chr(value);
           inc(k,3);
          end;
        end;
      end
     else
      begin
       s2:=s2+s[k]; inc(k);
      end;
    end;
  EndOfLoop:
  end;
 if s2<>'' then
  begin
   StringGrid1.RowCount:=i+1;
   StringGrid1.Cells[0,i]:=id;
   if FoForm.CStyle.Checked then s2:=FoForm.MakeCStyleString(s2);
   StringGrid1.Cells[1,i]:=s2;
   s2:='';
  end;
 CloseFile(f);
 with StringGrid1 do
  begin
   OrigString:=Cells[Col,Row];
   if FoForm.CStyle.Checked then
    CodeLab.Caption:=''
   else
    CodeLab.Caption:=FoForm.MakeCodes(Cells[Col,Row]);
  end;  
end;

function TInGameText.RunCompPrg: boolean;
var LastDatDate:TDateTime;
    s,s2:string;
begin
 Result:=false;
 DatUpdated:=false;
 try
  if FileExists(DatName) then
   LastDatDate:=FileDateToDateTime(FileAge(DatName))
  else
   LastDatDate:=0;
  case RunCompiler.ShowModal of
   mrOK: begin
          Result:=true;
          DatUpdated:=FileExists(DatName) and
                      (FileDateToDateTime(FileAge(DatName))>LastDatDate);
         end;
   mrCancel: begin
              s2:='mkpttxt.exe';
              SetLength(s,300);
              SetLength(s,FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM or FORMAT_MESSAGE_ARGUMENT_ARRAY,nil,
                        RunCompiler.ErrorCode,0,PChar(s),300,@s2));
              Application.MessageBox(PChar('An error occured while trying to run mkpttxt.exe:'#10#13+s),
                                     'Error',MB_ICONERROR or MB_ICONEXCLAMATION);
             end;
   end;
 except
  on E:Exception do
   begin
    Application.ShowException(E);
    Result:=false;
   end;
 end;

end;

procedure TInGameText.RunInit(var msg: TMessage);
var s:string;
begin
 PreviewTimer.Enabled:=FoForm.Preview.Checked;
 if FoForm.Preview.Checked then
  begin
   EnableWindow(PreviewForm.Handle,true);
   SetWindowPos(PreviewForm.Handle,HWND_TOPMOST,0,0,0,0,SWP_NOSIZE or SWP_NOMOVE or SWP_NOACTIVATE);
  end;
 If not FileExists(ExeName) then
  begin
   Application.MessageBox('Your TTD directory (The directory of the TTDPatch executable) doesn''t contain mkpttxt.exe.'#10#13+
                          'This file is needed to edit TTDPatch in-game texts.'#10#13+
                          'You can download it from the TTDPatch home page (http://www.ttdpatch.com).'#10#13+
                          'The in-game translator window will now close.',
                          'No in-game text compiler found',MB_ICONEXCLAMATION);
   Close;
   Exit;
  end;
 if FileExists(NewName) then
  begin
   if FileExists(TxtName) then
    begin
     s:='Your TTDPatch directory contains both ttdpttxt.txt and ttdpttxt.new'#10#13+
        'The .new file should contain new entries that was omitted from the .txt file';
     if DateTimeToFileDate(FileAge(TxtName))>DateTimeToFileDate(FileAge(NewName)) then
      s:=s+','#10#13'but the .txt file is more recent';
     s:=s+'.'#10#13'Do you want to use the .new file?'#10#13+
        '(Cancel will keep both files and close the in-game translator window)';
     case Application.MessageBox(PChar(s),'Updated file found',MB_ICONQUESTION or MB_YESNOCANCEL) of
      ID_YES: If not ((DeleteFile(TxtName)) and (RenameFile(NewName,TxtName))) then
               begin
                Application.MessageBox('File rename failed','Error',MB_ICONERROR);
                ModalResult:=mrCancel;
                Exit;
               end;
      ID_NO: If not DeleteFile(NewName) then
               begin
                Application.MessageBox('File delete failed','Error',MB_ICONERROR);
                ModalResult:=mrCancel;
                Exit;
               end;
      ID_CANCEL: begin
                  ModalResult:=mrCancel;
                  Exit;
                 end;
     end;
    end
   else RenameFile(NewName,TxtName);
  end;
 if RunCompPrg then
  begin
   if RunCompiler.ErrorCode<>0 then
    begin
     s:='The TTDPatch in-game text compiler reported an error';
     if FileExists(TxtName) then
      s:=s+', but you may continue because none of the necessary files are missing.'
     else
      begin
       s:=s+' and couldn''t create a necessary file (ttdpttxt.txt). The in-game translator window will close.';
       ModalResult:=mrCancel;
      end;
     RunLog.InfoText.Caption:=s;
     RunLog.Memo1.Text:=RunCompiler.ProgOutput;
     RunLog.ShowModal;
     if ModalResult<>mrNone then Exit;
    end;
  end
 else
  begin
   if not FileExists(TxtName) then
    begin
     Application.MessageBox('Can''t find a necessary file (ttdpttxt.txt). The in-game translator window will now close.','Error',
                            MB_ICONERROR);
     ModalResult:=MrCancel;
     Exit;
    end;
  end;
 CheckForNewFile;
 ReadTexts;
end;

procedure TInGameText.FormCreate(Sender: TObject);
begin
 StringGrid1.Cells[0,0]:='Identifier';
 StringGrid1.Cells[1,0]:='Text';
end;

procedure TInGameText.PreviewTimerTimer(Sender: TObject);
begin
 if not PreviewForm.Visible then PreviewTimer.Enabled:=false;
 With StringGrid1 do
  PreviewForm.PreviewText:=StringGrid1.Cells[Col,Row];
end;

procedure TInGameText.FormClose(Sender: TObject; var Action: TCloseAction);
begin
 PreviewTimer.Enabled:=false;
end;

procedure TInGameText.WriteTexts;
var f:TextFile;
    s,s2,s3:string;
    i,j:integer;
begin
 AssignFile(f,TxtName);
 Rewrite(f);
 Writeln(f,'// TTDPatch in-game text strings');
 Writeln(f,'// This file was written by TTD Translator 2.2');
 Writeln(f,'// If mkpttxt.exe can''t compile this file, look for an updated version of');
 Writeln(f,'// TTD Translator at http://www.csucsposta.hu/csaboka/ttdtrans.htm ,');
 Writeln(f,'// or send an e-mail to the author of TTD Translator: csaboka@programozo.hu');
 Writeln(f,'//');
 Writeln(f,'// TTDPatch and mkpttxt are Copyright (C) Joseph Drexler');
 Writeln(f,'//');
 for i:=1 to (StringGrid1.RowCount)-1 do
  begin
   s:='';
   if StringGrid1.Cells[0,i]<>'' then
    s:=StringGrid1.Cells[0,i]+'='
   else
    s:=#9;
   s:=s+'"';
   s2:=StringGrid1.Cells[1,i];
   if FoForm.CStyle.Checked then
    begin
     if FoForm.CompileCStyleString(s2,s3) then
      s2:=s3
     else
      s2:='';
    end;
   for j:=1 to Length(s2) do
    begin
     case s2[j] of
      #1..#12,#14..#31,#$7B..#$9E: s:=s+'\'+IntToHex(ord(s2[j]),2);
      #13: s:=s+'\n';
      '"': s:=s+'\"';
      '\': s:=s+'\\';
      else s:=s+s2[j];
     end;
     if ((s2[j]=' ') and (length(s)>70)) or (Length(s)>=80) then
      begin
       s:=s+'"';
       writeln(f,s);
       s:=#9'"';
      end;
    end;
   s:=s+'\00"';
   Writeln(f,s);
  end;
 CloseFile(f);
end;

procedure TInGameText.ApplicationEvents1Activate(Sender: TObject);
begin
 if not Visible then Exit;
 ShowWindow(Application.Handle,SW_RESTORE);
 SetFocus;
 if FoForm.Preview.Checked then
  SetWindowPos(PreviewForm.Handle,HWND_TOPMOST,0,0,0,0,SWP_NOSIZE or SWP_NOMOVE or SWP_NOACTIVATE);
end;

procedure TInGameText.ApplicationEvents1Deactivate(Sender: TObject);
begin
 if not Visible then Exit;
 if FoForm.Preview.Checked then
  begin
   SetWindowPos(PreviewForm.Handle,Handle,0,0,0,0,SWP_NOSIZE or SWP_NOMOVE or SWP_NOACTIVATE);
   SetWindowPos(Handle,PreviewForm.Handle,0,0,0,0,SWP_NOSIZE or SWP_NOMOVE or SWP_NOACTIVATE);
  end;
end;

procedure TInGameText.SaveClick(Sender: TObject);
var s:string;
begin
 if not CheckCError then Exit;
 WriteTexts;
 if RunCompPrg then
  begin
   if RunCompiler.ErrorCode<>0 then
    begin
     s:='The TTDPatch in-game text compiler reported an error';
     if DatUpdated then
      begin
       s:=s+', but it managed to compile a new .dat file anyway. You will be able to use the new in-game texts.';
       ModalResult:=mrOK;
      end
     else
      s:=s+' and couldn''t create a new .dat file. Your changes were saved, but you won''t see them when running TTDPatch.';
     RunLog.InfoText.Caption:=s;
     RunLog.Memo1.Text:=RunCompiler.ProgOutput;
     RunLog.ShowModal;
    end
   else ModalResult:=mrOK;
   if (ModalResult=mrOK) and CheckForNewFile then
    begin
     ModalResult:=mrNone;
     ReadTexts;
    end;
  end;
end;

function TInGameText.CheckCError: boolean;
{Check if the current cell in the string grid is correct. If it isn't, we show an error message
and return false.}
var s:string;
begin
 with StringGrid1 do
  if FoForm.CStyle.Checked and not FoForm.CompileCStyleString(Cells[Col,Row],s) then
   begin
    result:=false;
    Application.MessageBox(PChar('The C-style string you entered has an error:'#10#13+s+#10#13'You must fix this error to continue or press ESC to undo your changes'),'Error in string',MB_ICONERROR);
   end
  else
   result:=true;
end;

procedure TInGameText.StringGrid1SelectCell(Sender: TObject; ACol,
  ARow: Integer; var CanSelect: Boolean);
begin
 if not CheckCError then {If there's an error in the current cell, we don't allow to leave it}
  begin
   CanSelect:=false;
   Exit;
  end;
 if CanSelect then
  with StringGrid1 do
   begin
    if FoForm.CStyle.Checked then
     CodeLab.Caption:=''
    else
     CodeLab.Caption:=FoForm.MakeCodes(Cells[ACol,ARow]);
    OrigString:=Cells[ACol,ARow];
  end;
end;

procedure TInGameText.StringGrid1KeyPress(Sender: TObject; var Key: Char);
begin
 with StringGrid1 do
  if (Key=#13) and EditorMode and not CheckCError then
  {The user wants to exit editor mode by pressing Enter, but there's an error in the text,
   so we can't allow pressing Enter}
   Key:=#0
  else if (Key=#13) and not EditorMode then
  {The user enters editor mode, but we save the original text first, so we can restore it
   when the user presses Esc}
   OrigString:=Cells[Col,Row]
  else if Key=#27 then
  {The user presses Esc - we restore the original text}
   Cells[Col,Row]:=OrigString;
end;

procedure TInGameText.StringGrid1SetEditText(Sender: TObject; ACol,
  ARow: Integer; const Value: String);
begin
if FoForm.CStyle.Checked then
 CodeLab.Caption:=''
else
 CodeLab.Caption:=FoForm.MakeCodes(Value);
end;

procedure TInGameText.StringGrid1Exit(Sender: TObject);
begin
 if not CheckCError then StringGrid1.SetFocus; {If there's error in the current cell, the string grid
                                                prevents losing focus}
end;

function TInGameText.CheckForNewFile:boolean;
begin
 Result:=false;
 if FileExists(NewName) then
  case Application.MessageBox('The TTDPatch in-game text compiler created an updated text file'#10#13+
                              'that contains entries that were omitted from the original txt file.'#10#13+
                              'Do you want to use this updated file instead of your older one?'#10#13+
                              '(Yes is recommended)','Updated file found',MB_ICONQUESTION or MB_YESNO) of
   ID_YES: If not ((DeleteFile(TxtName)) and (RenameFile(NewName,TxtName))) then
            Application.MessageBox('File rename failed','Error',MB_ICONERROR)
           else Result:=true;
   ID_NO: If not DeleteFile(NewName) then
           Application.MessageBox('File delete failed','Error',MB_ICONERROR);
  end;
end;


procedure TInGameText.WMSysCommand(var Msg: TWMSysCommand);
begin
 if msg.CmdType=SC_MINIMIZE then
  begin
   SetWindowPos(Application.handle,Handle,Left,Top,Width,0,SWP_SHOWWINDOW);
   ShowWindow(Application.handle,SW_MINIMIZE);
  end
 else
  inherited;
end;

procedure TInGameText.HelpClick(Sender: TObject);
begin
 Application.HelpCommand(HELP_CONTEXT,10);
end;

end.
